/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <widget_app.h>

#include "main.h"
#include "view.h"
#include "data.h"
#include "log.h"

static void _btn_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static void _btn_up_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info);
static void _play_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _next_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _prev_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info);
static void _widget_clicked_cb(void *user_data);
static void _set_current_album(void *user_data, int index);

/*
 * @brief: The callback function is called after widget instance is created
 */
static int widget_instance_create(widget_context_h context, bundle *content, int w, int h, void *user_data)
{
	widget_instance_data_s *wid = (widget_instance_data_s *)malloc(sizeof(*wid));
	char *icon_path = NULL;
	char edj_path[PATH_MAX] = { 0, };

	if (content != NULL) {
		/* Recover the previous status with the bundle object. */
	}

	widget_app_context_set_tag(context, wid);

	/*
	 * Create basic GUI
	 */
	view_create(context, w, h);

	/*
	 * Get EDJ file path will be used for GUI
	 */
	data_get_resource_path(EDJ_FILE, edj_path, sizeof(edj_path));

	/*
	 * Create GUI for music player widget
	 */
	view_music_widget_create(context, edj_path);

	/*
	 * Set progressbar background image and color
	 */
	icon_path = data_get_image("sw.progressbar.bg");
	view_set_image(wid->layout, "sw.progressbar.bg", icon_path);
	view_set_color(wid->layout, "sw.progressbar.bg", 0, 87, 107, 255);
	free(icon_path);

	/*
	 * Set Play button and color
	 */
	icon_path = data_get_image("sw.icon.play");
	view_set_button(wid->layout, "sw.icon.play", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _play_btn_clicked_cb, wid);
	view_set_color(wid->layout, "sw.icon.play", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set Prev button and color
	 */
	icon_path = data_get_image("sw.icon.prev");
	view_set_button(wid->layout, "sw.icon.prev", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _prev_btn_clicked_cb, wid);
	view_set_color(wid->layout, "sw.icon.prev", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set Next button and color
	 */
	icon_path = data_get_image("sw.icon.next");
	view_set_button(wid->layout, "sw.icon.next", "focus", icon_path, NULL, _btn_down_cb, _btn_up_cb, _next_btn_clicked_cb, wid);
	view_set_color(wid->layout, "sw.icon.next", 250, 250, 250, 255);
	free(icon_path);

	/*
	 * Set progressbar that is used to display play time
	 */
	view_set_progressbar(wid->layout, "sw.progressbar", 53, 5);
	view_set_color_of_circle_object(wid->layout, "sw.progressbar", 0, 192, 235, 255*0.5);
	view_set_progressbar_val(wid->layout, "sw.progressbar", 70);

	/*
	 * Set first item in album list
	 */
	evas_object_data_set(wid->layout, "__DEVICE_INFO__", (void *) DEVICE_INFO_GEAR);
	evas_object_data_set(wid->layout, "__CURRENT_ALBUM_INDEX__", (void *)0);
	_set_current_album(wid, 0);

	/*
	 * Set the function will be called on the mouse events
	 */
	view_set_event_callback(wid->layout, "widget.event", WIDGET_EVENT_CLICKED, (void *) _widget_clicked_cb, wid);

	return WIDGET_ERROR_NONE;
}

/*
 * @brief: The callback function is called before widget instance is destroyed
 */
static int widget_instance_destroy(widget_context_h context, widget_app_destroy_type_e reason, bundle *content_info, void *user_data)
{
	void *wid = NULL;
	widget_app_context_get_tag(context, (void**) &wid);

	view_destroy(wid);

	return WIDGET_ERROR_NONE;
}

/*
 * @brief: The callback function is called when the widget is invisible
 */
static int widget_instance_pause(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes invisible. */
	return WIDGET_ERROR_NONE;
}

/*
 * @brief: The callback function is called when the widget is visible
 */
static int widget_instance_resume(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes visible. */
	return WIDGET_ERROR_NONE;
}

/*
 * @brief: The callback function is called when the event for updating widget is received
 */
static int widget_instance_update(widget_context_h context, bundle *content, int force, void *user_data)
{
	/* Take necessary actions when widget instance should be updated. */
	return WIDGET_ERROR_NONE;
}

/*
 * @brief: The callback function is called before the widget size is changed
 */
static int widget_instance_resize(widget_context_h context, int w, int h, void *user_data)
{
	void *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);
	view_resize(wid, w, h);

	return WIDGET_ERROR_NONE;
}

/*
 * @brief: This function handle widget instance language change event
 */
static void widget_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LANGUAGE_CHANGED */
	char *locale = NULL;
	app_event_get_language(event_info, &locale);
	elm_language_set(locale);
	free(locale);
}

/*
 * @brief: This function handle widget instance region change event
 */
static void widget_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_REGION_FORMAT_CHANGED */
}

/*
 * @brief: This function handle widget create event
 */
static widget_class_h widget_app_create(void *user_data)
{
	app_event_handler_h handlers[5] = {NULL, };

	widget_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED],
			APP_EVENT_LANGUAGE_CHANGED, widget_app_lang_changed, user_data);
	widget_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED],
			APP_EVENT_REGION_FORMAT_CHANGED, widget_app_region_changed,
			user_data);

	widget_instance_lifecycle_callback_s ops = {
			.create = widget_instance_create,
			.destroy = widget_instance_destroy,
			.pause = widget_instance_pause,
			.resume = widget_instance_resume,
			.update = widget_instance_update,
			.resize = widget_instance_resize,
	};

	return widget_app_class_create(ops, user_data);
}

/*
 * @brief: This callback function is called once after the main loop of the application exits
 */
static void widget_app_terminate(void *user_data)
{
	/* Release all resources. */
}

/*
 * @brief: Main function of application
 */
int main(int argc, char *argv[])
{
	widget_app_lifecycle_callback_s ops = {0, };
	int ret;

	ops.create = widget_app_create;
	ops.terminate = widget_app_terminate;

	ret = widget_app_main(argc, argv, &ops, NULL);
	if (ret != WIDGET_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "widget_app_main() is failed. err = %d", ret);
	}

	return ret;
}

/*
 * @brief: The function operated on widget click
 * @param[user_data]: Structure has informations for managing this widget
 */
static void _widget_clicked_cb(void *user_data)
{
	Evas_Event_Flags event_flag;
	widget_instance_data_s *wid = user_data;

	/* Check whether this event is CLICK or MOUSE UP */
	event_flag = evas_event_default_flags_get(evas_object_evas_get(wid->win));
	/* Click event on scroll, we'll not deal with */
	if (event_flag == EVAS_EVENT_FLAG_ON_HOLD) {
		return;
	}

	dlog_print(DLOG_DEBUG, LOG_TAG, "clicked a widget.");
}

/*
 * @brief: The function operated on button down
 * @param[user_data]: Data to pass to function when it is called
 * @param[e]: The canvas where the object is on
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _btn_down_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info)
{
	evas_object_color_set(obj, 250, 250, 250, 102);
}

/*
 * @brief: The function operated on button up
 * @param[user_data]: Data to pass to function when it is called
 * @param[e]: The canvas where the object is on
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _btn_up_cb(void *user_data, Evas *e, Evas_Object *obj, void *event_info)
{
	evas_object_color_set(obj, 250, 250, 250, 255);
}

/*
 * @brief: The function operated on play button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _play_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "play button is clicked");
}

/*
 * @brief: The function operated on next button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _next_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "next button is clicked");
}

/*
 * @brief: The function operated on prev button click
 * @param[user_data]: Data to pass to function when it is called
 * @param[obj]: Object that event is triggered
 * @param[event_info]: Informations regarding with event
 */
static void _prev_btn_clicked_cb(void *user_data, Evas_Object *obj, void *event_info)
{
	dlog_print(DLOG_DEBUG, LOG_TAG, "prev button is clicked");
}

/*
 * @brief: Set current album information(album art, title, artist, device icon)
 * @param[user_data]: The user data to be passed to this functions
 * @param[index]: The order of album list(0 is the first item in album list)
 */
static void _set_current_album(void *user_data, int index)
{
	widget_instance_data_s *wid  = user_data;
	char *album_art = NULL;
	char *title = NULL;
	char *artist = NULL;
	char *icon = NULL;
	char default_img_path[PATH_MAX] = { 0, };
	char edj_path[PATH_MAX] = { 0, };
	device_info_e device;

	data_get_resource_path("images/music_no_album_art.png", default_img_path, (int) PATH_MAX);
	data_get_resource_path(EDJ_FILE, edj_path, (int) PATH_MAX);

	album_art = data_get_album_art();
	view_music_set_album_art(wid->layout, "sw.bg", album_art, default_img_path);

	device = (device_info_e) evas_object_data_get(wid->layout, "__DEVICE_INFO__");
	icon = data_get_device_image(device);
	title = data_get_album_title();
	view_music_set_title_layout(wid->layout, "sw.title.layout", edj_path, icon, title, wid);
	free(icon);

	artist = data_get_album_artist();
	view_set_text(wid->layout, "txt.artist", artist);
}
