/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <widget_app.h>
#include <widget_app_efl.h>
#include <dlog.h>
#include "log.h"
#include "view.h"

#define LABEL_STYLE_START "<font=Tizen:style=Regular><font_size=36><align=center><color=#FAFAFA><wrap=mixed>"
#define LABEL_STYLE_END "</wrap></color></align></font_size></font>"

static void _widget_pressed_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info);
static void _widget_released_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info);

/*
 * @brief: Set title layout that will be shown
 * @param[parent]: Object has part to which you want to set
 * @param[part_name]: Part name to which you want to set
 * @param[edj_path]: edje path to which you want to set
 * @param[image_path]: path of image file you want to set
 * @param[title]: text you want to set
 * @param[user_data]: data needed in this function
 */
void view_music_set_title_layout(Evas_Object *parent, const char *part_name, const char *edj_path, const char *image_path, const char *title, void *user_data)
{
	Evas_Object *title_layout = NULL;

	title_layout = elm_object_part_content_get(parent, part_name);
	if (title_layout == NULL) {
		title_layout = view_create_layout(parent, edj_path, "title-layout", NULL, user_data);
		elm_object_part_content_set(parent, part_name, title_layout);
	}

	if (image_path) {
		view_set_image(title_layout, "sw.icon.device", image_path);
		view_set_color(title_layout, "sw.icon.device", 250, 250, 250, 255);
	}

	if (title) {
		Evas_Object *label = elm_object_part_content_get(title_layout, "sw.title");
		if (label == NULL) {
			view_set_label(title_layout, "sw.title");
		}
		view_set_label_text(title_layout, "sw.title", title);
	}
}

/*
 * @brief: Set album art image that will be shown
 * @param[parent]: Object has part to which you want to set
 * @param[part_name]: Part name to which you want to set
 * @param[image_path]: Path of album art file you want to set
 * @param[default_image_path]: Path of default album art file
 */
void view_music_set_album_art(Evas_Object *parent, const char *part_name, const char *image_path, const char *default_image_path)
{
	if (!strcmp(image_path, "NULL")) {
		view_set_image(parent, "sw.bg", default_image_path);
	} else {
		view_set_image(parent, part_name, image_path);
	}
}

/*
 * @brief: Set image to given part
 * @param[parent]: Object has part to which you want to set this image
 * @param[part_name]: Part name to which you want to set this image
 * @param[image_path]: Path of image file
 */
void view_set_image(Evas_Object *parent, const char *part_name, const char *image_path)
{
	Evas_Object *image = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	image = elm_object_part_content_get(parent, part_name);
	if (image == NULL) {
		image = elm_image_add(parent);
		if (!image) {
			dlog_print(DLOG_ERROR, LOG_TAG, "failed to create an image object.");
			return;
		}
	}

	if (EINA_FALSE == elm_image_file_set(image, image_path, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to set image.");
		return;
	}

	if (part_name) {
		elm_object_part_content_set(parent, part_name, image);
	} else {
		elm_object_content_set(parent, image);
	}

	evas_object_show(image);

	return;
}

/*
 * @brief: Set text to the part
 * @param[parent]: Object has part to which you want to set text
 * @param[part_name]: Part name to which you want to set text
 * @param[text]: Text you want to set to the part
 */
void view_set_text(Evas_Object *parent, const char *part_name, const char *text)
{
	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	/* Set text of target part object */
	elm_object_part_text_set(parent, part_name, text);
}

/*
 * @brief: Set color of the part
 * @param[parent]: Object has part to which you want to set color
 * @param[part_name]: Name of part to which you want to set color
 * @param[r]: R of RGBA you want to set to the part
 * @param[g]: G of RGBA you want to set to the part
 * @param[b]: B of RGBA you want to set to the part
 * @param[a]: A of RGBA you want to set to the part
 */
void view_set_color(Evas_Object *parent, const char *part_name, int r, int g, int b, int a)
{
	Evas_Object *obj = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	obj = elm_object_part_content_get(parent, part_name);
	if (obj == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get parent.");
		return;
	}

	/* Set color of target part object */
	evas_object_color_set(obj, r, g, b, a);
}

/*
 * @brief: Set the color of circle line and font in a given circle object
 * @param[parent]: Object has part to which you want to set color
 * @param[part_name]: Name of part to which you want to set color
 * @param[r] The red component of the given color
 * @param[g] The green component of the given color
 * @param[b] The blue component of the given color
 * @param[a] The alpha component of the given color
 */
void view_set_color_of_circle_object(Evas_Object *parent, const char *part_name, int r, int g, int b, int a)
{
	Evas_Object *obj = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	obj = elm_object_part_content_get(parent, part_name);
	if (obj == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get parent.");
		return;
	}

	/* Set color of target part object */
	eext_circle_object_color_set(obj, r, g, b, a);
}

/*
 * @brief: Make a label and set label options.
 * @param[parent]: The object to which you want to add this label
 */
Evas_Object *view_create_label(Evas_Object *parent)
{
	Evas_Object *label = elm_label_add(parent);

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	elm_object_style_set(label, "slide_short");
	elm_label_wrap_width_set(label, 230);
	elm_label_ellipsis_set(label, EINA_TRUE);
	elm_label_slide_duration_set(label, 2);
	elm_label_slide_mode_set(label, ELM_LABEL_SLIDE_MODE_NONE);
	elm_label_slide_go(label);
	evas_object_show(label);

	return label;
}

/*
 * @brief: Set a text to label object
 * @param[parent]: Object has part to which you want to set text
 * @param[part_name]: Part name to which you want to set text
 * @param[text]: Text you want to set to the part
 */
void view_set_label_text(Evas_Object *parent, const char *part_name, const char *text)
{
	Evas_Object *label = NULL;
	char *markup_text = NULL;
	char buf[256] = { 0, };

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	markup_text = elm_entry_utf8_to_markup(text);
	snprintf(buf, sizeof(buf), "%s%s%s", LABEL_STYLE_START, markup_text, LABEL_STYLE_END);
	free(markup_text);

	label = elm_object_part_content_get(parent, part_name);
	if (label == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "label is NULL.");
		return;
	}

	elm_object_text_set(label, buf);
}

/*
 * @brief: Set a label to given part
 * @param[parent]: Object has part to which you want to set this label
 * @param[part_name]: Part name to which you want to set this label
 */
void view_set_label(Evas_Object *parent, const char *part_name)
{
	Evas_Object *label = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	label = view_create_label(parent);
	if (label == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "label is NULL.");
		return;
	}

	elm_object_part_content_set(parent, part_name, label);
}

/*
 * @brief: Create a progressbar
 * @param[parent]: Object has part to which you want to set this progressbar
 * @param[radius]: The radius value of a given circle object
 * @param[line_width]: The line width value of the circle object
 */
Evas_Object *view_create_progressbar(Evas_Object *parent, int radius, int line_width)
{
	Evas_Object *progressbar = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	progressbar = eext_circle_object_progressbar_add(parent, NULL);

	eext_circle_object_value_min_max_set(progressbar, 0.0, 100.0);
	eext_circle_object_radius_set(progressbar, radius);
	eext_circle_object_line_width_set(progressbar, line_width);
	evas_object_show(progressbar);

	return progressbar;
}

/*
 * @brief: Set a progressbar to given part
 * @param[parent]: Object has part to which you want to set
 * @param[part_name]: Part name to which you want to set
 * @param[radius]: The radius value of a given circle object
 * @param[line_width]: The line width value of the circle object
 */
void view_set_progressbar(Evas_Object *parent, const char *part_name, int radius, int line_width)
{
	Evas_Object *progressbar = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	progressbar = view_create_progressbar(parent, radius, line_width);
	if (progressbar == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	elm_object_part_content_set(parent, part_name, progressbar);
}

/*
 * @brief: Set value to a progressbar
 * @param[parent]: Object has part to which you want to set
 * @param[part_name]: Part name to which you want to set
 * @param[val]: Value to which you want to set
 */
void view_set_progressbar_val(Evas_Object *parent, const char *part_name, int val)
{
	Evas_Object *progressbar = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	if (part_name == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "part is NULL.");
		return;
	}

	progressbar = elm_object_part_content_get(parent, part_name);
	if (progressbar == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "progressbar is NULL.");
		return;
	}

	eext_circle_object_value_set(progressbar, val);
}

/*
 * @brief: Make and set button.
 * @param[parent]: Object to which you want to set the button
 * @param[style]: Style of the button
 * @param[text]: Text will be written on the button
 * @param[image_path]: Path of image file will be used as button icon
 * @param[part_name]: Part name in EDJ to which you want to set the button
 * @param[down_cb]: Function will be operated when the button is pressed
 * @param[up_cb]: Function will be operated when the button is released
 * @param[clicked_cb]: Function will be operated when the button is clicked
 * @param[user_data]: Data needed in this function
 */
void view_set_button(Evas_Object *parent, const char *part_name, const char *style, const char *image_path, const char *text,
		Evas_Object_Event_Cb down_cb, Evas_Object_Event_Cb up_cb, Evas_Smart_Cb clicked_cb, void *user_data)
{
	Evas_Object *btn = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	btn = elm_button_add(parent);
	if (!btn) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create button.");
		return;
	}

	if (style)
		elm_object_style_set(btn, style);

	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(parent, part_name, btn);

	if (text)
		elm_object_text_set(btn, text);

	if (image_path)
		view_set_image(btn, "icon", image_path);

	if (down_cb)
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_DOWN, down_cb, user_data);
	if (up_cb)
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, up_cb, user_data);
	if (clicked_cb)
		evas_object_smart_callback_add(btn, "clicked", clicked_cb, user_data);

	evas_object_show(btn);
}

/*
 * @brief: Make a layout to target parent object with edje file
 * @param[parent]: The object to which you want to add this layout
 * @param[edj_path]: File path of EDJ
 * @param[group_name]: Name of group in EDJ you want to set to
 * @param[cb_function]: Callback function will be operated when back key is pressed
 * @param[user_data]: The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *edj_path, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	/* Create layout by EDC(edje file) */
	layout = elm_layout_add(parent);
	elm_layout_file_set(layout, edj_path, group_name);

	/* Layout size setting */
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (cb_function)
		eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, cb_function, user_data);

	evas_object_show(layout);

	return layout;
}

/*
 * @brief: Make and set a layout to conformant
 * @param[parent]: Target conformant object
 * @param[file_name]: File path of EDJ used
 * @param[group_name]: Group name in EDJ you want to set to layout
 * @param[cb_function]: Callback for back event handling
 * @param[user_data]: The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout_for_conformant(Evas_Object *parent, const char *file_name, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	/* Create layout for conformant */
	layout = view_create_layout(parent, file_name, group_name, cb_function, user_data);
	if (layout == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "layout is NULL.");
		return NULL;
	}

	elm_object_content_set(parent, layout);

	return layout;
}

/*
 * This is an empty layout.
 * You need to append detailed contents to content.edc and view.c.
 * If you need more views, please feel free to add new layouts.
 */
void view_music_widget_create(widget_context_h context, const char *edj_path)
{
	widget_instance_data_s *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);

	wid->layout = view_create_layout_for_conformant(wid->conform, edj_path, GROUP_NAME, NULL, wid);
	if (wid->layout == NULL) {
		evas_object_del(wid->win);
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a content.");
		return;
	}

	return;
}

/*
 * @brief: Create basic window for the widget with w, h size
 * @param[context]: The context of widget instance
 * @param[w]: The pixel value for widget width
 * @param[h]: The pixel value for widget height
 */
Evas_Object *view_create_widget_win(widget_context_h context, int w, int h)
{
	Evas_Object *win = NULL;
	int ret;

	if (context == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get context.");
		return NULL;
	}

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate window
	 */
	ret = widget_app_get_elm_win(context, &win);
	if (ret != WIDGET_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get window. err = %d", ret);
		return NULL;
	}

	evas_object_resize(win, w, h);
	evas_object_show(win);

	return win;
}

/*
 * @brief: Make conformant without indicator for wearable app.
 * @param[win]: Window to which you want to set this conformant
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);
	evas_object_show(conform);

	return conform;
}

/*
 * @brief: Create Essential Object window, conformant
 * @param[context]: The context of widget instance
 * @param[w]: The pixel value for widget width
 * @param[h]: The pixel value for widget height
 */
void view_create(widget_context_h context, int w, int h)
{
	widget_instance_data_s *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);

	wid->win = view_create_widget_win(context, w, h);

	/*
	 * Create conformant
	 */
	wid->conform = view_create_conformant_without_indicator(wid->win);
	if (wid->conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return;
	}

	wid->root_width = w;
	wid->root_height = h;
}

/*
 * @brief: Destroy the view
 * @param[user_data]: Structure has informations for managing this widget
 */
void view_destroy(void *user_data)
{
	widget_instance_data_s *wid = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to destroy data.");
		return;
	}

	evas_object_del(wid->win);

	free(wid);
}


/*
 * @brief: Resize the window
 * @param[user_data]: Structure has informations for managing this widget
 * @param[w]: The pixel value for widget width
 * @param[h]: The pixel value for widget height
 */
void view_resize(void *user_data, int w, int h)
{
	widget_instance_data_s *wid = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to widget data.");
		return;
	}

	evas_object_resize(wid->win, w, h);
}

/*
 * @brief: Set the function will be called on the mouse events
 * @param[parent]: Layout will be received the event
 * @param[part_name]: Part name that the event is triggered
 * @param[event_type]: Event type
 * @param[event_cb]: The function will be called on the given event
 * @param[user_data]: The data will be passed to the event_cb
 */
void view_set_event_callback(Evas_Object *parent, const char *part_name, widget_event_type event_type, void *event_cb, void *user_data)
{
	Evas_Object *obj = NULL;

	obj = elm_object_part_content_get(parent, part_name);

	if (!obj) {
		obj = (Evas_Object*) edje_object_part_object_get(elm_layout_edje_get(parent), part_name);
	}

	evas_object_data_set(obj, "callback_func_addr", (void*) event_cb);
	evas_object_data_set(obj, "widget_event_type", (void*) event_type);

	if (event_type == WIDGET_EVENT_PRESSED) {
		evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_DOWN, _widget_pressed_cb, user_data);
	} else if ((event_type == WIDGET_EVENT_RELEASED) || (event_type == WIDGET_EVENT_CLICKED)) {
		evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_UP, _widget_released_cb, user_data);
	}
}

/*
* @brief: The function will be called when widget is pressed
* @param[data]: Structure has informations for managing this widget
* @param[evas]: Evas object
* @param[obj]: Object that the event is triggered
* @param[event_info]: The information regarding with the event
*/
static void _widget_pressed_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	widget_instance_data_s *wid = data;
	if (!wid) {
		return;
	}

	widget_event_cb func = evas_object_data_get(obj, "callback_func_addr");

	func(wid);
}

/*
 * @brief: The function will be called when widget is released
 * @param[data]: Structure has informations for managing this widget
 * @param[evas]: Evas object
 * @param[obj]: Object that the event is triggered
 * @param[event_info]: The information regarding with the event
 */
static void _widget_released_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	widget_instance_data_s *wid = data;
	Evas_Event_Mouse_Up *ev = event_info;
	Evas_Coord x, y, w, h;

	if (!wid) {
		return;
	}

	widget_event_cb func = (widget_event_cb) evas_object_data_get(obj, "callback_func_addr");
	widget_event_type type = (widget_event_type) evas_object_data_get(obj, "widget_event_type");

	if (type == WIDGET_EVENT_RELEASED) {
		if (func) {
			func(wid);
		}
	} else if (type == WIDGET_EVENT_CLICKED) {
		evas_object_geometry_get(obj, &x, &y, &w, &h);

		if (!ELM_RECTS_POINT_OUT(x, y, w, h, ev->canvas.x, ev->canvas.y)) {
			if (func) {
				func(wid);
			}
		}
	}
}
